#####################################################################################
#                  pyShodan: Python API Wrapper for Shodan                          #
#                       Copyright (c) 2019 GoVanguard                               #
#####################################################################################
# This file is part of pyShodan.                                                    #
#                                                                                   #
#     pyShodan is free software: you can redistribute it and/or modify              #
#     it under the terms of the GNU Lesser General Public License as published by   #
#     the Free Software Foundation, either version 3 of the License, or             #
#     (at your option) any later version.                                           #
#                                                                                   #
#     pyShodan is distributed in the hope that it will be useful,                   #
#     but WITHOUT ANY WARRANTY; without even the implied warranty of                #
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
#     GNU Lesser General Public License for more details.                           #
#                                                                                   #
#     You should have received a copy of the GNU Lesser General Public License      #
#     along with pyShodan.  If not, see <http://www.gnu.org/licenses/>.             #
#####################################################################################

from pyShodan import pyShodan
import sys
import argparse
import csv
import time
import datetime

def getApiKey(k):
    api = k
    return api

def writeFile(t, h):
    with open(t,"w") as csvfile:
        header = ["Host IP", "FQDN", "Banner", "Ports"]
        writer = csv.writer(csvfile, delimiter=",")
        writer.writerow(header)
        for i in range(len(h)):
            writer.writerow(h[i])
    print(t + " created in script directory")

def searchTerm(s):
    searchStr = s
    api = getApiKey(args.apiKey)
    ps = pyShodan(api, False)
    hostinfo = ps.searchTerm(searchStr)
    title = "shodanOutput-" + searchStr + ".csv"
    writeFile(title, hostinfo)

def searchIp(d):
    searchHost = d
    api = getApiKey(args.apiKey)
    ps = pyShodan(api, False)
    try:
        hostinfo = ps.searchIp(searchHost)
        title = "shodanOutput-" + searchHost + ".csv"
        writeFile(title, hostinfo)
    except shodan.APIError as e:
        print("Error: %s" % e)

def searchList(f):
    api = getApiKey(args.apiKey)
    ps = pyShodan(api, False)
    d = datetime.datetime.today()
    hostinfo = ps.searchList(f)
    title = "shodanOutput-ipList_" + d.strftime("%d-%m-%Y_%H-%M-%S") + ".csv"
    writeFile(title, hostinfo)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="pyShodan test script")

    parser.add_argument("--search", "-s", action="store", type=str, dest="searchVal", help="Search Shodan for a general term")
    parser.add_argument("--ipaddr", "-ip", action="store", type=str, dest="ipSearch", help="Search Shodan for a specific IP address")
    parser.add_argument("--iplist", action="store", dest="ipList", help="Search Shodan for every IP address in a txt file")
    parser.add_argument("--api", "-api", action="store", type=str, dest="apiKey", required=True, help="Shodan API key")
    args = parser.parse_args()

    if args.searchVal:
        searchTerm(args.searchVal)
    elif args.ipSearch:
        searchIp(args.ipSearch)
    elif args.ipList:
        searchList(args.ipList)
    elif args.apiKey:
        getApiKey(apiKey)
    else:
        print("Invalid arguments, see -h for details. Example use: python pyShodan.py -s SearchTerm -ip IPAddress -api SHODAN-api-key")



#####################################################################################
#                  pyShodan: Python API Wrapper for Shodan                          #
#                       Copyright (c) 2019 GoVanguard                               #
#####################################################################################
# This file is part of pyShodan.                                                    #
#                                                                                   #
#     pyShodan is free software: you can redistribute it and/or modify              #
#     it under the terms of the GNU Lesser General Public License as published by   #
#     the Free Software Foundation, either version 3 of the License, or             #
#     (at your option) any later version.                                           #
#                                                                                   #
#     pyShodan is distributed in the hope that it will be useful,                   #
#     but WITHOUT ANY WARRANTY; without even the implied warranty of                #
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                 #
#     GNU Lesser General Public License for more details.                           #
#                                                                                   #
#     You should have received a copy of the GNU Lesser General Public License      #
#     along with pyShodan.  If not, see <http://www.gnu.org/licenses/>.             #
#####################################################################################

import shodan
import sys
import time

class pyShodan:

    #Constructor
    def __init__(self, apiToken: str, debug=False):
        self.__api = shodan.Shodan(apiToken)
        self.__debug = debug

    def debug(self, val: bool):
        self.__debug = val

    def searchTerm(self, searchStr: str):
        api = self.__api
        hostinfo = []

        # Search Shodan for this term
        results = api.search(searchStr)

        # Format the results into list
        print('Results found: %s' % results['total'])
        for result in results['matches']:
            hostinfo.append([result['ip_str'].replace(","," "), result['data'].replace(","," ").encode("utf-8"),result['port']]) # Store the results in a list

        return hostinfo

    def searchIp(self, searchHost: str):
        api = self.__api
        hostinfo = []

        # Search Shodan for this IP address
        try:
            host = api.host(searchHost)

            for item in host['data']:
                hostinfo.append([item['ip_str'], item['org'], str(item['data'].replace(',',' ').strip('\t\n\r')), item['port']]) # Store the results in a list

        except shodan.APIError as e:
            print("Error: %s" % e)

        return hostinfo

    def searchList(self, f: str):
        api = self.__api
        hostinfo = []

        # Iterate through lines in the file
        for i in range(len(f.read().splitlines())):
            try:
                time.sleep(2)
                host = api.host(x[i]) # Search Shodan for the host on the current line in the file
                for item in host['data']:
                    hostinfo.append([item['ip_str'], item['org'], str(item['data']).replace(',',' ').strip('\r\n\t'), item['port']]) # Store the results in a list
            except shodan.APIError as e:
                print("Error: %s" % e)
                if "no information available" in str(e).lower():
                    print("No information is available for %s" % str(x[i]))

        return hostinfo



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyShodan",
    version="0.2.0",
    author="Shane Scott",
    author_email="sscott@gvit.com",
    description="Python library for querying the Shodan API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/GoVanguard/pyShodan",
    packages=['pyShodan'],
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
    ),
)
